@@ -1,29 +1,26 @@
 import os
-import subprocess
+
 import click
 import numpy as np
 from osgeo import gdal
 
-from overflow.breach_single_cell_pits import breach_single_cell_pits
-from overflow.flow_direction import flow_direction
-from overflow.breach_paths_least_cost import (
-    breach_paths_least_cost,
-    breach_paths_least_cost_cuda,
-)
-from overflow.fix_flats.tiled import fix_flats_tiled
-from overflow.fix_flats.core import fix_flats_from_file
-from overflow.fill_depressions.core import fill_depressions
-from overflow.fill_depressions.tiled import fill_depressions_tiled
-from overflow.flow_accumulation.core import flow_accumulation
-from overflow.flow_accumulation.tiled import flow_accumulation_tiled
-from overflow.basins.core import label_watersheds_from_file, drainage_points_from_file
+from overflow.basins.core import drainage_points_from_file, label_watersheds_from_file
 from overflow.basins.tiled import label_watersheds_tiled
+from overflow.breach_paths_least_cost import breach_paths_least_cost
+from overflow.breach_single_cell_pits import breach_single_cell_pits
 from overflow.extract_streams.core import extract_streams
 from overflow.extract_streams.tiled import extract_streams_tiled
+from overflow.fill_depressions.core import fill_depressions
+from overflow.fill_depressions.tiled import fill_depressions_tiled
+from overflow.fix_flats.core import fix_flats_from_file
+from overflow.fix_flats.tiled import fix_flats_tiled
+from overflow.flow_accumulation.core import flow_accumulation
+from overflow.flow_accumulation.tiled import flow_accumulation_tiled
+from overflow.flow_direction import flow_direction
+from overflow.util.cli_progress import RichProgressDisplay
 from overflow.util.constants import DEFAULT_CHUNK_SIZE, DEFAULT_SEARCH_RADIUS
-from overflow.util.raster import sqmi_to_cell_count, feet_to_cell_count
-from overflow.util.timer import timer, console, resource_stats
-
+from overflow.util.raster import feet_to_cell_count, sqmi_to_cell_count
+from overflow.util.timer import console, resource_stats, timer
 
 # set gdal configuration
 gdal.UseExceptions()
@@ -49,27 +46,26 @@ def breach_single_cell_pits_cli(input_file: str, output_file: str, chunk_size: i
     This function is used to breach single cell pits in a DEM.
     The function takes filepath to a GDAL supported raster dataset as
     input and prodeces an output DEM with breached single cell pits.
-
-    Parameters
-    ----------
-    input_file : str
-        Path to the input dem file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        breach_single_cell_pits(input_file, output_file, chunk_size)
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Breaching single cell pits"):
+            with timer("Breach single cell pits", spinner=False):
+                breach_single_cell_pits(
+                    input_file, output_file, chunk_size, progress_display.callback
+                )
+                resource_stats.add_output_file("Breached DEM", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(
-            f"breach_single_cell_pits failed with the following exception: {str(exc)}"
+        console.print(
+            f"[bold red]Error:[/bold red] breach_single_cell_pits failed with the following exception: {str(exc)}"
         )
-        raise click.Abort()  # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
+        raise click.Abort()
 
 
 @main.command(name="flow-direction")
@@ -83,25 +79,25 @@ def flow_direction_cli(input_file: str, output_file: str, chunk_size: int):
     """
     This function is used to generate flow direction rasters from chunks of a DEM.
     The function takes a chunk of a DEM as input and returns a chunk of DEM with delineated flow direction.
-
-    Parameters
-    ----------
-    input_file : str
-        Path to the input dem file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        flow_direction(input_file, output_file, chunk_size)
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Computing flow direction"):
+            with timer("Flow direction", spinner=False):
+                flow_direction(
+                    input_file, output_file, chunk_size, progress_display.callback
+                )
+                resource_stats.add_output_file("Flow Direction", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"flow_direction failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] flow_direction failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -118,60 +114,42 @@ def flow_direction_cli(input_file: str, output_file: str, chunk_size: int):
     help="maximum cost of breach paths (total sum of elevation removed from each cell in path)",
     default=np.inf,
 )
-@click.option(
-    "--cuda",
-    help="use experimental CUDA implementation",
-    is_flag=True,
-)
-@click.option(
-    "--max_pits",
-    help="maximum number of pits to process in each chunk for cuda implementation",
-    default=10000,
-)
 def breach_paths_least_cost_cli(
     input_file: str,
     output_file: str,
     chunk_size: int,
     search_radius: int,
     max_cost: float,
-    cuda: bool,
-    max_pits: int,
 ):
     """
     This function is used to breach paths of least cost for pits in a DEM.
     The function takes filepath to a GDAL supported raster dataset as
     input and prodeces an output DEM with breached paths of least cost.
     Only pits that can be solved within the search radius are solved.
-
-    Parameters
-    ----------
-    input_file : str
-        Path to the input dem file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-    search_radius : int
-        Search radius in cells to look for solution paths. Larger search radius will use more memory.
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        if cuda:
-            breach_paths_least_cost_cuda(
-                input_file, output_file, chunk_size, search_radius, max_pits, max_cost
-            )
-        else:
-            breach_paths_least_cost(
-                input_file, output_file, chunk_size, search_radius, max_cost
-            )
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Breaching paths (least cost)"):
+            with timer("Breach paths least cost", spinner=False):
+                breach_paths_least_cost(
+                    input_file,
+                    output_file,
+                    chunk_size,
+                    search_radius,
+                    max_cost,
+                    progress_display.callback,
+                )
+                resource_stats.add_output_file("Breached DEM", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(
-            f"breach_paths_least_cost failed with the following exception: {str(exc)}"
+        console.print(
+            f"[bold red]Error:[/bold red] breach_paths_least_cost failed with the following exception: {str(exc)}"
         )
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -211,30 +189,33 @@ def fix_flats_cli(
     This function is used to fix flats in a DEM.
     The function takes filepath to a GDAL supported raster dataset as
     input and prodeces an output DEM with fixed flats.
-
-    Parameters
-    ----------
-    dem_file : str
-        Path to the input dem file
-    fdr_file : str
-        Path to the input flow direction file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        if chunk_size <= 1:
-            fix_flats_from_file(dem_file, fdr_file, output_file)
-        else:
-            fix_flats_tiled(dem_file, fdr_file, output_file, chunk_size, working_dir)
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Fixing flats"):
+            with timer("Fix flats", spinner=False):
+                if chunk_size <= 1:
+                    fix_flats_from_file(dem_file, fdr_file, output_file)
+                else:
+                    fix_flats_tiled(
+                        dem_file,
+                        fdr_file,
+                        output_file,
+                        chunk_size,
+                        working_dir,
+                        progress_display.callback,
+                    )
+                resource_stats.add_output_file("Fixed FDR", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"fix_flats failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] fix_flats failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -274,36 +255,33 @@ def fill_depressions_cli(
     This function is used to fill depressions in a DEM.
     The function takes filepath to a GDAL supported raster dataset as
     input and prodeces an output DEM with filled depressions.
-
-    Parameters
-    ----------
-    dem_file : str
-        Path to the input dem file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-        If chunk_size is less than or equal to 1, the fill_depressions function is called
-        which fills depressions with an all in RAM algorithm.
-    working_dir : str
-        Working directory to store temporary files
-    fill_holes : bool
-        If set, fills holes in the DEM
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        if chunk_size <= 1:
-            fill_depressions(dem_file, output_file, fill_holes)
-        else:
-            fill_depressions_tiled(
-                dem_file, output_file, chunk_size, working_dir, fill_holes
-            )
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Filling depressions"):
+            with timer("Fill depressions", spinner=False):
+                if chunk_size <= 1:
+                    fill_depressions(dem_file, output_file, fill_holes)
+                else:
+                    fill_depressions_tiled(
+                        dem_file,
+                        output_file,
+                        chunk_size,
+                        working_dir,
+                        fill_holes,
+                        progress_display.callback,
+                    )
+                resource_stats.add_output_file("Filled DEM", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"fill_depressions failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] fill_depressions failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -331,30 +309,28 @@ def flow_accumulation_cli(
     """
     This function is used to calculate flow accumulation from a flow direction raster.
     The function takes a flow direction raster as input and returns a flow accumulation raster.
-
-    Parameters
-    ----------
-    fdr_file : str
-        Path to the input flow direction file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-        If chunk_size is less than or equal to 1, the flow_accumulation function is called
-        which calculates flow accumulation with an all in RAM algorithm.
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        if chunk_size <= 1:
-            flow_accumulation(fdr_file, output_file)
-        else:
-            flow_accumulation_tiled(fdr_file, output_file, chunk_size)
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Computing flow accumulation"):
+            with timer("Flow accumulation", spinner=False):
+                if chunk_size <= 1:
+                    flow_accumulation(fdr_file, output_file)
+                else:
+                    flow_accumulation_tiled(
+                        fdr_file, output_file, chunk_size, progress_display.callback
+                    )
+                resource_stats.add_output_file("Flow Accumulation", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"flow_accumulation failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] flow_accumulation failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -401,38 +377,39 @@ def label_watersheds_cli(
     """
     This function is used to label watersheds from a flow direction raster.
     The function takes a flow direction raster and drainage points as input and returns a watersheds raster.
-
-    Parameters
-    ----------
-    fdr_file : str
-        Path to the input flow direction file
-    dp_file : str
-        Path to the drainage points file
-    output_file : str
-        Path to the output file
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-        If chunk_size is less than or equal to 1, the label_watersheds function is called
-        which calculates watersheds with an all in RAM algorithm.
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        # TODO, snap the drainage points to the flow accumulation grid
-        if chunk_size <= 1:
-            label_watersheds_from_file(
-                fdr_file, dp_file, output_file, all_basins, dp_layer
-            )
-        else:
-            drainage_points = drainage_points_from_file(fdr_file, dp_file, dp_layer)
-            label_watersheds_tiled(
-                fdr_file, drainage_points, output_file, chunk_size, all_basins
-            )
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Labeling watersheds"):
+            with timer("Label watersheds", spinner=False):
+                # TODO, snap the drainage points to the flow accumulation grid
+                if chunk_size <= 1:
+                    label_watersheds_from_file(
+                        fdr_file, dp_file, output_file, all_basins, dp_layer
+                    )
+                else:
+                    drainage_points = drainage_points_from_file(
+                        fdr_file, dp_file, dp_layer
+                    )
+                    label_watersheds_tiled(
+                        fdr_file,
+                        drainage_points,
+                        output_file,
+                        chunk_size,
+                        all_basins,
+                        progress_display.callback,
+                    )
+                resource_stats.add_output_file("Watersheds", output_file)
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"label_watersheds failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] label_watersheds failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -472,36 +449,39 @@ def extract_streams_cli(
     """
     This function is used to extract streams from a flow accumulation and flow direction raster.
     The function takes a flow accumulation and flow direction raster as input and returns a streams raster.
-
-    Parameters
-    ----------
-    fac_file : str
-        Path to the input flow accumulation file
-    fdr_file : str
-        Path to the input flow direction file
-    output_dir : str
-        Path to the output directory
-    cell_count_threshold : int
-        Cell count threshold
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-        If chunk_size is less than or equal to 1, the extract_streams function is called
-        which extracts streams with an all in RAM algorithm.
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
-        if chunk_size <= 1:
-            extract_streams(fac_file, fdr_file, output_dir, cell_count_threshold)
-        else:
-            extract_streams_tiled(
-                fac_file, fdr_file, output_dir, cell_count_threshold, chunk_size
-            )
+        progress_display = RichProgressDisplay()
+        with progress_display.progress_context("Extracting streams"):
+            with timer("Extract streams", spinner=False):
+                if chunk_size <= 1:
+                    extract_streams(
+                        fac_file,
+                        fdr_file,
+                        output_dir,
+                        cell_count_threshold,
+                        progress_display.callback,
+                    )
+                else:
+                    extract_streams_tiled(
+                        fac_file,
+                        fdr_file,
+                        output_dir,
+                        cell_count_threshold,
+                        chunk_size,
+                        progress_display.callback,
+                    )
+                resource_stats.add_output_file("Streams", f"{output_dir}/streams.gpkg")
+                success = True
+
+        console.print(resource_stats.get_summary_panel(success=success))
     except Exception as exc:
-        print(f"extract_streams failed with the following exception: {str(exc)}")
-        # exit with non-zero exit code. Everytime zero is returned on failure a baby kitten dies
+        console.print(
+            f"[bold red]Error:[/bold red] extract_streams failed with the following exception: {str(exc)}"
+        )
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
@@ -560,34 +540,11 @@ def process_dem_cli(
     """
     This function is used to process a DEM.
     The function takes a DEM as input and returns a streams raster.
-
-    Parameters
-    ----------
-    dem_file : str
-        Path to the input dem file
-    output_dir : str
-        Path to the output directory
-    chunk_size : int
-        Size of the chunk to be used for processing. Larger chunk sizes will use more memory.
-        If chunk_size is 0 or negative, core algorithms will be used instead of tiled processing
-        (except for breaching which will use the larger of the raster dimensions as chunk size).
-    search_radius_ft : float
-        Search radius in feet to look for solution paths. Larger search radius will use more memory.
-    max_cost : float
-        Maximum cost of breach paths (total sum of elevation removed from each cell in path)
-    da_sqmi : float
-        Minimum drainage area in square miles for stream extraction
-    basins : bool
-        Flag to enable watershed delineation
-    fill_holes : bool
-        If set, fills holes in the DEM
-
-    Returns
-    -------
-    None
     """
+    success = False
     try:
         console.rule("[bold blue]DEM Processing")
+        progress_display = RichProgressDisplay()
 
         with timer("Total processing", silent=True, spinner=False):
             search_radius = feet_to_cell_count(search_radius_ft, dem_file)
@@ -618,175 +575,166 @@ def process_dem_cli(
             console.print(f"  Fill holes: [cyan]{fill_holes}[/cyan]")
 
             if search_radius > 0:
-                with timer("Breaching", spinner=False):
-                    breach_paths_least_cost(
-                        dem_file,
+                with progress_display.progress_context("Breaching paths"):
+                    with timer("Breaching", spinner=False):
+                        breach_paths_least_cost(
+                            dem_file,
+                            f"{output_dir}/dem_corrected.tif",
+                            core_chunk_size,
+                            search_radius,
+                            max_cost,
+                            progress_display.callback,
+                        )
+
+                with progress_display.progress_context("Filling depressions"):
+                    with timer("Filling", spinner=False):
+                        if chunk_size <= 0:
+                            fill_depressions(
+                                f"{output_dir}/dem_corrected.tif", None, fill_holes
+                            )
+                        else:
+                            fill_depressions_tiled(
+                                f"{output_dir}/dem_corrected.tif",
+                                None,
+                                chunk_size,
+                                output_dir,
+                                fill_holes,
+                                progress_display.callback,
+                            )
+            else:
+                with progress_display.progress_context("Filling depressions"):
+                    with timer("Filling", spinner=False):
+                        if chunk_size <= 0:
+                            fill_depressions(
+                                dem_file, f"{output_dir}/dem_corrected.tif", fill_holes
+                            )
+                        else:
+                            fill_depressions_tiled(
+                                dem_file,
+                                f"{output_dir}/dem_corrected.tif",
+                                chunk_size,
+                                output_dir,
+                                fill_holes,
+                                progress_display.callback,
+                            )
+
+            resource_stats.add_output_file(
+                "Corrected DEM", f"{output_dir}/dem_corrected.tif"
+            )
+
+            with progress_display.progress_context("Computing flow direction"):
+                with timer("Flow direction", spinner=False):
+                    flow_direction(
                         f"{output_dir}/dem_corrected.tif",
+                        f"{output_dir}/fdr.tif",
                         core_chunk_size,
-                        search_radius,
-                        max_cost,
+                        progress_display.callback,
                     )
 
-                with timer("Filling", spinner=False):
+            resource_stats.add_output_file("Flow Direction", f"{output_dir}/fdr.tif")
+
+            with progress_display.progress_context("Fixing flats"):
+                with timer("Fixing flats", spinner=False):
                     if chunk_size <= 0:
-                        fill_depressions(
-                            f"{output_dir}/dem_corrected.tif", None, fill_holes
+                        fix_flats_from_file(
+                            f"{output_dir}/dem_corrected.tif",
+                            f"{output_dir}/fdr.tif",
+                            None,
                         )
                     else:
-                        fill_depressions_tiled(
+                        fix_flats_tiled(
                             f"{output_dir}/dem_corrected.tif",
+                            f"{output_dir}/fdr.tif",
                             None,
                             chunk_size,
                             output_dir,
-                            fill_holes,
+                            progress_display.callback,
                         )
-            else:
-                with timer("Filling", spinner=False):
+
+            with progress_display.progress_context("Computing flow accumulation"):
+                with timer("Flow accumulation", spinner=False):
                     if chunk_size <= 0:
-                        fill_depressions(
-                            dem_file, f"{output_dir}/dem_corrected.tif", fill_holes
+                        flow_accumulation(
+                            f"{output_dir}/fdr.tif", f"{output_dir}/accum.tif"
                         )
                     else:
-                        fill_depressions_tiled(
-                            dem_file,
-                            f"{output_dir}/dem_corrected.tif",
+                        flow_accumulation_tiled(
+                            f"{output_dir}/fdr.tif",
+                            f"{output_dir}/accum.tif",
                             chunk_size,
-                            output_dir,
-                            fill_holes,
+                            progress_display.callback,
                         )
 
-            with timer("Flow direction", spinner=False):
-                flow_direction(
-                    f"{output_dir}/dem_corrected.tif",
-                    f"{output_dir}/fdr.tif",
-                    core_chunk_size,
-                )
+            resource_stats.add_output_file(
+                "Flow Accumulation", f"{output_dir}/accum.tif"
+            )
 
-            with timer("Fixing flats", spinner=False):
-                if chunk_size <= 0:
-                    fix_flats_from_file(
-                        f"{output_dir}/dem_corrected.tif", f"{output_dir}/fdr.tif", None
-                    )
-                else:
-                    fix_flats_tiled(
-                        f"{output_dir}/dem_corrected.tif",
-                        f"{output_dir}/fdr.tif",
-                        None,
-                        chunk_size,
-                        output_dir,
-                    )
+            with progress_display.progress_context("Extracting streams"):
+                with timer("Stream extraction", spinner=False):
+                    if chunk_size <= 0:
+                        extract_streams(
+                            f"{output_dir}/accum.tif",
+                            f"{output_dir}/fdr.tif",
+                            output_dir,
+                            threshold,
+                            progress_display.callback,
+                        )
+                    else:
+                        extract_streams_tiled(
+                            f"{output_dir}/accum.tif",
+                            f"{output_dir}/fdr.tif",
+                            output_dir,
+                            threshold,
+                            chunk_size,
+                            progress_display.callback,
+                        )
 
-            with timer("Flow accumulation", spinner=False):
-                if chunk_size <= 0:
-                    flow_accumulation(
-                        f"{output_dir}/fdr.tif", f"{output_dir}/accum.tif"
-                    )
-                else:
-                    flow_accumulation_tiled(
-                        f"{output_dir}/fdr.tif", f"{output_dir}/accum.tif", chunk_size
-                    )
-
-            with timer("Stream extraction", spinner=False):
-                if chunk_size <= 0:
-                    extract_streams(
-                        f"{output_dir}/accum.tif",
-                        f"{output_dir}/fdr.tif",
-                        output_dir,
-                        threshold,
-                    )
-                else:
-                    extract_streams_tiled(
-                        f"{output_dir}/accum.tif",
-                        f"{output_dir}/fdr.tif",
-                        output_dir,
-                        threshold,
-                        chunk_size,
-                    )
+            resource_stats.add_output_file("Streams", f"{output_dir}/streams.gpkg")
 
             if basins:
-                with timer("Watershed delineation", spinner=False):
-                    drainage_points = drainage_points_from_file(
-                        f"{output_dir}/fdr.tif",
-                        f"{output_dir}/streams.gpkg",
-                        "junctions",
-                    )
-
-                    if chunk_size <= 0:
-                        label_watersheds_from_file(
+                with progress_display.progress_context("Labeling watersheds"):
+                    with timer("Watershed delineation", spinner=False):
+                        drainage_points = drainage_points_from_file(
                             f"{output_dir}/fdr.tif",
                             f"{output_dir}/streams.gpkg",
-                            f"{output_dir}/basins.tif",
-                            False,
                             "junctions",
                         )
-                    else:
-                        label_watersheds_tiled(
-                            f"{output_dir}/fdr.tif",
-                            drainage_points,
-                            f"{output_dir}/basins.tif",
-                            chunk_size,
-                            False,
-                        )
+
+                        if chunk_size <= 0:
+                            label_watersheds_from_file(
+                                f"{output_dir}/fdr.tif",
+                                f"{output_dir}/streams.gpkg",
+                                f"{output_dir}/basins.tif",
+                                False,
+                                "junctions",
+                            )
+                        else:
+                            label_watersheds_tiled(
+                                f"{output_dir}/fdr.tif",
+                                drainage_points,
+                                f"{output_dir}/basins.tif",
+                                chunk_size,
+                                False,
+                                progress_display.callback,
+                            )
+
+                resource_stats.add_output_file("Basins", f"{output_dir}/basins.tif")
+
+            success = True
 
         console.rule("[bold green]Processing Complete")
-
-        # Display resource summary chart
-        console.print(resource_stats.get_chart())
+        console.print(resource_stats.get_summary_panel(success=True))
 
     except Exception as exc:
         console.print(
             f"[bold red]Error:[/bold red] process_dem failed with the following exception: {str(exc)}"
         )
-        raise click.Abort()
-
-
-@main.command(name="test")
-@click.option(
-    "--all",
-    "run_all",
-    help="Run all tests including CUDA tests",
-    is_flag=True,
-)
-@click.argument("pytest_args", nargs=-1, type=click.UNPROCESSED)
-def test_cli(run_all: bool, pytest_args: tuple):
-    """
-    Run pytest tests. By default, excludes CUDA tests with -k "not cuda".
-    Use --all to run all tests including CUDA tests.
-    Additional pytest arguments can be passed after the command options.
-
-    Examples:
-        overflow test                    # Run tests excluding CUDA
-        overflow test --all              # Run all tests including CUDA
-        overflow test tests/             # Run specific test directory
-        overflow test -v                 # Run with verbose output
-    """
-    try:
-        cmd = ["pytest"]
-
-        # Add -k "not cuda" flag by default unless --all is specified
-        if not run_all:
-            cmd.extend(["-k", "not cuda"])
-
-        # Add any additional pytest arguments
-        if pytest_args:
-            cmd.extend(pytest_args)
-
-        # Run pytest
-        result = subprocess.run(cmd, cwd=os.getcwd())
-
-        # Exit with the same code as pytest
-        raise SystemExit(result.returncode)
-
-    except FileNotFoundError:
-        print("Error: pytest not found. Make sure pytest is installed.")
-        raise click.Abort()
-    except Exception as exc:
-        print(f"test command failed with the following exception: {str(exc)}")
+        if not success:
+            console.print(resource_stats.get_summary_panel(success=False))
         raise click.Abort()
 
 
 if __name__ == "__main__":
     # run the function
-    # pylint does not understand click decorators
-    # pylint: disable=no-value-for-parameter
+
     main()
