@@ -1,38 +1,38 @@
-import os
 import concurrent.futures
-import time
-import sys
-from threading import Lock
+import os
 import queue
-from shapely.geometry import LineString
-from shapely.ops import linemerge
-from shapely.wkt import loads, dumps
-from rich.console import Console
-import numpy as np
-from numba import njit, types
-from numba.typed import Dict, List  # pylint: disable=no-name-in-module
-from numba.types import int64, float64
+import time
+from threading import Lock
+
 import numba
+import numpy as np
+from numba import njit, types  # type: ignore[attr-defined]
+from numba.typed import Dict, List  # type: ignore[attr-defined]
+from numba.types import float64, int64
 from osgeo import gdal, ogr
+from shapely.geometry import LineString
+from shapely.wkt import dumps, loads
+
+from overflow.extract_streams.core import (
+    add_downstream_junctions,
+    find_node_cells,
+    get_downstream_cell,
+    get_stream_raster,
+    nodes_to_points,
+    setup_datasource,
+    write_lines,
+    write_points,
+)
 from overflow.util.constants import NEIGHBOR_OFFSETS
+from overflow.util.progress import ProgressCallback, ProgressTracker, silent_callback
 from overflow.util.raster import (
-    create_dataset,
-    open_dataset,
-    raster_chunker,
     RasterChunk,
     cell_to_coords,
     coords_to_cell,
+    create_dataset,
     grid_hash,
-)
-from overflow.extract_streams.core import (
-    get_downstream_cell,
-    find_node_cells,
-    get_stream_raster,
-    setup_datasource,
-    write_points,
-    write_lines,
-    nodes_to_points,
-    add_downstream_junctions,
+    open_dataset,
+    raster_chunker,
 )
 
 # Define custom types for numba
@@ -343,13 +343,14 @@ def remove_tile_edge_junctions(
     streams_dataset_path: str,
     streams_layer: str = "streams",
     junctions_layer: str = "junctions",
+    progress_callback: ProgressCallback | None = None,
 ):
     """
     Remove junctions that have exactly 2 stream endpoints coincident with the junction
     and merge the connecting stream segments.
     """
-    console = Console()
-    is_a_tty = sys.stdout.isatty()
+    if progress_callback is None:
+        progress_callback = silent_callback
 
     ds = ogr.Open(streams_dataset_path, gdal.GA_Update)
     streams_layer = ds.GetLayer(streams_layer)
@@ -359,97 +360,111 @@ def remove_tile_edge_junctions(
     junction_points = []
     junction_fids = []
     for feature in junctions_layer:
-        geom = feature.GetGeometryRef()
+        geom = feature.GetGeometryRef()  # type: ignore[attr-defined]
         junction_points.append((geom.GetX(), geom.GetY()))
-        junction_fids.append(feature.GetFID())
+        junction_fids.append(feature.GetFID())  # type: ignore[attr-defined]
 
-    junction_points = np.array(junction_points, dtype=np.float64)
-    junction_fids = np.array(junction_fids, dtype=np.int64)
+    junction_points = np.array(junction_points, dtype=np.float64)  # type: ignore[assignment]
+    junction_fids = np.array(junction_fids, dtype=np.int64)  # type: ignore[assignment]
 
     # Collect all stream endpoints
     stream_endpoints = []
     for feature in streams_layer:
-        fid = feature.GetFID()
-        geom = feature.GetGeometryRef()
+        fid = feature.GetFID()  # type: ignore[attr-defined]
+        geom = feature.GetGeometryRef()  # type: ignore[attr-defined]
         upstream = geom.GetPoint(0)
         downstream = geom.GetPoint(geom.GetPointCount() - 1)
         # Store both endpoints with flag indicating if it's upstream
         stream_endpoints.append((fid, upstream[0], upstream[1], 1))
         stream_endpoints.append((fid, downstream[0], downstream[1], 0))
 
-    stream_endpoints = np.array(stream_endpoints, dtype=np.float64)
+    stream_endpoints = np.array(stream_endpoints, dtype=np.float64)  # type: ignore[assignment]
 
     # Get lists of what needs to be merged/deleted
-    with console.status("[bold green]Finding streams to merge...") as status:
-        junction_fids_to_remove, stream_pairs_to_merge, new_endpoints = (
-            find_streams_to_merge(
-                junction_points, junction_fids, stream_endpoints, geotransform
-            )
+    progress_callback(
+        phase="Finding streams to merge",
+        step=1,
+        total_steps=3,
+        progress=0.33,
+        message="Analyzing stream connections",
+    )
+    junction_fids_to_remove, stream_pairs_to_merge, new_endpoints = (
+        find_streams_to_merge(
+            junction_points, junction_fids, stream_endpoints, geotransform
         )
+    )
 
     # Process the results with GDAL
     total = len(junction_fids_to_remove)
 
     # Keep track of stream replacements
-    stream_replacements = {}  # old_fid -> new_fid
+    stream_replacements: dict[int, int] = {}  # old_fid -> new_fid
 
-    with console.status("[bold green]Merging streams...") as status:
-        for i, ((fid1, fid2, merge_type), (up_x, up_y, down_x, down_y)) in enumerate(
-            zip(stream_pairs_to_merge, new_endpoints), 1
-        ):
-            if is_a_tty:
-                status.update(f"[bold green]Merging streams: {i}/{total}")
+    for i, (
+        (fid1, fid2, merge_type),
+        (_up_x, _up_y, _down_x, _down_y),
+    ) in enumerate(zip(stream_pairs_to_merge, new_endpoints), 1):
+        # Report progress
+        progress_callback(
+            phase="Merging streams",
+            step=i,
+            total_steps=total,
+            progress=(i / total) * 0.33 + 0.33,
+            message=f"Merging stream {i}/{total}",
+        )
 
-            # Get current FIDs accounting for previous merges
-            current_fid1 = stream_replacements.get(fid1, fid1)
-            current_fid2 = stream_replacements.get(fid2, fid2)
+        # Get current FIDs accounting for previous merges
+        current_fid1 = stream_replacements.get(fid1, fid1)
+        current_fid2 = stream_replacements.get(fid2, fid2)
 
-            # Get and merge the streams
-            stream1 = streams_layer.GetFeature(current_fid1)
-            stream2 = streams_layer.GetFeature(current_fid2)
+        # Get and merge the streams
+        stream1 = streams_layer.GetFeature(current_fid1)  # type: ignore[attr-defined]
+        stream2 = streams_layer.GetFeature(current_fid2)  # type: ignore[attr-defined]
 
-            if stream1 is None or stream2 is None:
-                console.print(
-                    f"[yellow]Warning: Could not find streams {current_fid1} and/or {current_fid2}, skipping..."
-                )
-                continue
+        if stream1 is None or stream2 is None:
+            # Skip this merge - warning would clutter output
+            continue
 
-            geom1 = stream1.GetGeometryRef().Clone()
-            geom2 = stream2.GetGeometryRef().Clone()
+        geom1 = stream1.GetGeometryRef().Clone()
+        geom2 = stream2.GetGeometryRef().Clone()
 
-            merged_geom = merge_stream_geometries(geom1, geom2, merge_type)
+        merged_geom = merge_stream_geometries(geom1, geom2, merge_type)
 
-            # Create new feature
-            new_feature = ogr.Feature(streams_layer.GetLayerDefn())
-            new_feature.SetGeometry(merged_geom)
-            streams_layer.CreateFeature(new_feature)
-            new_fid = new_feature.GetFID()
+        # Create new feature
+        new_feature = ogr.Feature(streams_layer.GetLayerDefn())  # type: ignore[attr-defined]
+        new_feature.SetGeometry(merged_geom)
+        streams_layer.CreateFeature(new_feature)  # type: ignore[attr-defined]
+        new_fid = new_feature.GetFID()
 
-            # Update stream replacements
-            stream_replacements[fid1] = new_fid
-            stream_replacements[fid2] = new_fid
+        # Update stream replacements
+        stream_replacements[fid1] = new_fid
+        stream_replacements[fid2] = new_fid
 
-            # For any streams that were previously merged into fid1 or fid2,
-            # update their replacements to point to the new FID
-            for old_fid, replacement_fid in list(stream_replacements.items()):
-                if replacement_fid in (current_fid1, current_fid2):
-                    stream_replacements[old_fid] = new_fid
+        # For any streams that were previously merged into fid1 or fid2,
+        # update their replacements to point to the new FID
+        for old_fid, replacement_fid in list(stream_replacements.items()):
+            if replacement_fid in (current_fid1, current_fid2):
+                stream_replacements[old_fid] = new_fid
 
-            # Delete original features
-            streams_layer.DeleteFeature(current_fid1)
-            streams_layer.DeleteFeature(current_fid2)
+        # Delete original features
+        streams_layer.DeleteFeature(current_fid1)  # type: ignore[attr-defined]
+        streams_layer.DeleteFeature(current_fid2)  # type: ignore[attr-defined]
 
-            # Cleanup
-            stream1 = None
-            stream2 = None
-            new_feature = None
+        # Cleanup
+        stream1 = None
+        stream2 = None
+        new_feature = None
 
     # Remove junctions
-    with console.status("[bold green]Removing junctions...") as status:
-        for i, junction_fid in enumerate(junction_fids_to_remove, 1):
-            if is_a_tty:
-                status.update(f"[bold green]Removing junctions: {i}/{total}")
-            junctions_layer.DeleteFeature(junction_fid)
+    for i, junction_fid in enumerate(junction_fids_to_remove, 1):
+        progress_callback(
+            phase="Removing junctions",
+            step=i,
+            total_steps=total,
+            progress=(i / total) * 0.33 + 0.67,
+            message=f"Removing junction {i}/{total}",
+        )
+        junctions_layer.DeleteFeature(junction_fid)  # type: ignore[attr-defined]
 
     # Cleanup
     ds = None
@@ -461,7 +476,8 @@ def extract_streams_tiled(
     output_dir: str,
     cell_count_threshold: int,
     chunk_size: int,
-):
+    progress_callback: ProgressCallback | None = None,
+) -> None:
     """
     Extract stream networks from flow accumulation and flow direction rasters using a tiled approach.
 
@@ -479,7 +495,17 @@ def extract_streams_tiled(
         output_dir (str): Directory to save output files.
         cell_count_threshold (int): Minimum flow accumulation to be considered a stream.
         chunk_size (int): Size of each tile for processing.
+        progress_callback (ProgressCallback | None): Optional callback for progress updates.
+            If None, the operation runs silently.
+
+    Returns:
+        None
     """
+    # Setup progress tracking
+    if progress_callback is None:
+        progress_callback = silent_callback
+    tracker = ProgressTracker(progress_callback, "Extract Streams", total_steps=3)
+
     # Open input datasets
     fac_ds = open_dataset(fac_path)
     fdr_ds = open_dataset(fdr_path)
@@ -508,8 +534,8 @@ def extract_streams_tiled(
     )
 
     # Set up parallel processing
-    max_workers = numba.config.NUMBA_NUM_THREADS  # pylint: disable=no-member
-    task_queue = queue.Queue(max_workers)
+    max_workers = numba.config.NUMBA_NUM_THREADS  # type: ignore[attr-defined]
+    task_queue: queue.Queue[int] = queue.Queue(max_workers)
     lock = Lock()
 
     def handle_tile_result(future):
@@ -534,11 +560,13 @@ def extract_streams_tiled(
             write_lines(lines_layer, lines)
             task_queue.get()
 
-    print("Step 1 of 3: Extracting stream networks from tiles")
+    tracker.update(1, "Extracting stream networks from tiles")
 
     # Process tiles in parallel
     with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-        for fac_tile in raster_chunker(fac_band, chunk_size):
+        for fac_tile in raster_chunker(
+            fac_band, chunk_size, progress_callback=progress_callback
+        ):
             while task_queue.full():
                 time.sleep(0.1)
             task_queue.put(0)
@@ -570,10 +598,14 @@ def extract_streams_tiled(
     fac_ds = None
     fdr_ds = None
 
-    print("Step 2 of 3: Merging stream segments across tiles")
+    tracker.update(2, "Merging stream segments across tiles")
 
-    remove_tile_edge_junctions(geotransform, streams_dataset_path)
+    remove_tile_edge_junctions(
+        geotransform, streams_dataset_path, progress_callback=progress_callback
+    )
 
-    print("Step 3 of 3: Adding downstream junctions")
+    tracker.update(3, "Adding downstream junctions")
 
-    add_downstream_junctions(geotransform, streams_dataset_path)
+    add_downstream_junctions(
+        geotransform, streams_dataset_path, progress_callback=progress_callback
+    )
