@@ -1,12 +1,14 @@
+from collections.abc import Generator
 from enum import Enum
-import sys
-from typing import Generator, Any
-from osgeo import gdal, osr
+from typing import Any
+
 import numpy as np
-from rich.console import Console
-from numba import int64, float32, njit
+from numba import float32, int64, njit  # type: ignore[attr-defined]
 from numba.experimental import jitclass
+from osgeo import gdal, osr
+
 from overflow.util.constants import NEIGHBOR_OFFSETS
+from overflow.util.progress import ProgressCallback, silent_callback
 
 gdal.UseExceptions()
 
@@ -36,7 +38,7 @@ def gdal_data_type_to_numpy_data_type(gdal_dtype: int) -> np.dtype:
         "CFloat32": np.complex64,
         "CFloat64": np.complex128,
     }
-    return gdal_numpy_dtype_mapping[gdal.GetDataTypeName(gdal_dtype)]
+    return gdal_numpy_dtype_mapping[gdal.GetDataTypeName(gdal_dtype)]  # type: ignore[return-value]
 
 
 def read_raster_with_bounds_handling(
@@ -117,7 +119,7 @@ class RasterChunk:
         size: int,
         buffer_size: int,
     ):
-        self.data = None
+        self.data: np.ndarray | None = None
         self.row = row
         self.col = col
         self.size = size
@@ -149,6 +151,7 @@ class RasterChunk:
         )
 
     def _get_unbuffered_data(self, band: gdal.Band) -> np.ndarray:
+        assert self.data is not None
         y_remaining = max(band.YSize - self.row * self.size, 0)
         x_remaining = max(band.XSize - self.col * self.size, 0)
         unbuffered_y_size = min(self.size, y_remaining)
@@ -176,16 +179,21 @@ class RasterChunk:
 
 
 def raster_chunker(
-    band: gdal.Band, chunk_size: int, chunk_buffer_size: int = 0, lock=None
+    band: gdal.Band,
+    chunk_size: int,
+    chunk_buffer_size: int = 0,
+    lock=None,
+    progress_callback: ProgressCallback | None = None,
 ) -> Generator[RasterChunk, Any, None]:
     """Generator that yields chunks of a raster.
 
     Args:
-        band  (gdal.Band): The raster band to read from.
-        chunk_row_size (int): The number of rows in each chunk.
-        chunk_col_size (int): The number of columns in each chunk.
-        buffer_row_size (int): The number of rows in the buffer region.
-        buffer_col_size (int): The number of columns in the buffer region.
+        band (gdal.Band): The raster band to read from.
+        chunk_size (int): The size of each chunk in pixels.
+        chunk_buffer_size (int): The size of the buffer region around each chunk.
+        lock: Optional threading lock for concurrent access.
+        progress_callback (ProgressCallback | None): Optional callback for progress updates.
+            If None, the operation runs silently.
 
     Yields:
         Generator[RasterChunk]: A generator that yields a RasterChunk.
@@ -195,36 +203,38 @@ def raster_chunker(
     n_chunks_col = (band.XSize + chunk_size - 1) // chunk_size
     # Calculate the total number of chunks
     total_chunks = n_chunks_row * n_chunks_col
-    console = Console()
-    is_a_tty = sys.stdout.isatty()
+
+    # Use silent callback if none provided
+    if progress_callback is None:
+        progress_callback = silent_callback
+
     # Iterate over the chunks
-    with console.status("[bold green]Processing Chunks: ") as status:
-        for chunk_row in range(n_chunks_row):
-            for chunk_col in range(n_chunks_col):
-                # Read the chunk and yield it
-                chunk = RasterChunk(
-                    chunk_row,
-                    chunk_col,
-                    chunk_size,
-                    chunk_buffer_size,
-                )
-                if lock:
-                    with lock:
-                        chunk.read(band)
-                else:
+    for chunk_row in range(n_chunks_row):
+        for chunk_col in range(n_chunks_col):
+            chunk_index = chunk_row * n_chunks_col + chunk_col + 1
+
+            # Report progress
+            progress_callback(
+                phase="Processing Chunks",
+                step=chunk_index,
+                total_steps=total_chunks,
+                progress=chunk_index / total_chunks,
+                message=f"Chunk {chunk_index}/{total_chunks}",
+            )
+
+            # Read the chunk and yield it
+            chunk = RasterChunk(
+                chunk_row,
+                chunk_col,
+                chunk_size,
+                chunk_buffer_size,
+            )
+            if lock:
+                with lock:
                     chunk.read(band)
-                yield chunk
-                # Log progress
-                if is_a_tty:
-                    status.update(
-                        f"[bold green]Processing Chunks: {chunk_row * n_chunks_col + chunk_col + 1}/{total_chunks}"
-                    )
-                else:
-                    print(
-                        f"Processing Chunks: {chunk_row * n_chunks_col + chunk_col + 1}/{total_chunks}",
-                        end="\r",
-                        flush=True,
-                    )
+            else:
+                chunk.read(band)
+            yield chunk
 
 
 def create_grid_cell_class(value_type):
@@ -232,9 +242,9 @@ def create_grid_cell_class(value_type):
     class GridCell:
         """A class to represent a cell in the grid. Used with heapq to prioritize cells by cost."""
 
-        row: int64
-        col: int64
-        value: value_type
+        row: int64  # type: ignore[annotation-unchecked]
+        col: int64  # type: ignore[annotation-unchecked]
+        value: value_type  # type: ignore[annotation-unchecked]
 
         def __init__(self, row, col, value):
             self.row = row
@@ -319,7 +329,7 @@ def get_tile_perimeter(array: np.ndarray) -> np.ndarray:
 
 
 @njit
-def neighbor_generator(row, col, n_row, n_col):
+def neighbor_generator(row: int, col: int, n_row: int, n_col: int):
     """
     A generator function that yields the coordinates of the neighbors of a given cell in a 2D grid.
 
@@ -381,8 +391,8 @@ def create_dataset(
     data_type: int,
     x_size: int,
     y_size: int,
-    geotransform: tuple = None,
-    projection: str = None,
+    geotransform: tuple | None = None,
+    projection: str | None = None,
 ) -> gdal.Dataset:
     """
     Creates a new raster dataset with the specified parameters using ZSTD compression.
@@ -426,7 +436,7 @@ def create_dataset(
         raise ValueError(f"Error creating dataset '{filepath}': {e}") from e
 
 
-def read_tile(filepath: str):
+def read_tile(filepath: str) -> np.ndarray:
     """
     Reads a raster tile from the specified file path and returns its data as a numpy array.
 
@@ -438,7 +448,7 @@ def read_tile(filepath: str):
     """
     ds = open_dataset(filepath)
     band = ds.GetRasterBand(1)
-    return band.ReadAsArray()
+    return band.ReadAsArray()  # type: ignore[no-any-return]
 
 
 def mosaic_tiles(
@@ -452,7 +462,7 @@ def mosaic_tiles(
     y_size: int,
     geotransform: tuple,
     projection: str,
-):
+) -> None:
     """
     Mosaics multiple raster tiles into a single raster dataset.
 
@@ -545,8 +555,8 @@ class TileManager:
             tile_data = self.cache[tile_key]
             offset_x = tile_col * self.tile_size
             offset_y = tile_row * self.tile_size
-            tile_width = min(self.tile_size, self.raster_width - offset_x)
-            tile_height = min(self.tile_size, self.raster_height - offset_y)
+            min(self.tile_size, self.raster_width - offset_x)
+            min(self.tile_size, self.raster_height - offset_y)
 
             self.band.WriteArray(tile_data, offset_x, offset_y)
             self.dirty_tiles.remove(tile_key)
@@ -586,9 +596,9 @@ def cell_to_coords(
     i: np.int32,
     j: np.int32,
     gt: np.ndarray,
-    tile_row: np.int32 = 0,
-    tile_col: np.int32 = 0,
-    chunk_size: np.int32 = 0,
+    tile_row: np.int32 = 0,  # type: ignore[assignment]
+    tile_col: np.int32 = 0,  # type: ignore[assignment]
+    chunk_size: np.int32 = 0,  # type: ignore[assignment]
 ) -> tuple[np.float64, np.float64]:
     """
     Convert raster cell indices to geographic coordinates using numerically stable calculations.
@@ -624,9 +634,9 @@ def coords_to_cell(
     x: np.float64,
     y: np.float64,
     gt: np.ndarray,
-    tile_row: np.int32 = 0,
-    tile_col: np.int32 = 0,
-    chunk_size: np.int32 = 0,
+    tile_row: np.int32 = 0,  # type: ignore[assignment]
+    tile_col: np.int32 = 0,  # type: ignore[assignment]
+    chunk_size: np.int32 = 0,  # type: ignore[assignment]
 ) -> tuple[np.int32, np.int32]:
     """
     Convert geographic coordinates to raster cell indices using numerically stable calculations.
@@ -696,7 +706,7 @@ def grid_hash(i: int, j: int) -> int:
     a = np.uint64(2 * abs(i) if i >= 0 else 2 * abs(i) - 1)
     b = np.uint64(2 * abs(j) if j >= 0 else 2 * abs(j) - 1)
     c = np.int64((a * a + a + b if a >= b else a + b * b) // 2)
-    return c if (i < 0 and j < 0) or (i >= 0 and j >= 0) else -c - 1
+    return c if (i < 0 and j < 0) or (i >= 0 and j >= 0) else -c - 1  # type: ignore[return-value]
 
 
 def get_units_to_meters_conversion(raster_ds: gdal.Dataset) -> float:
@@ -767,13 +777,13 @@ def get_units_to_meters_conversion(raster_ds: gdal.Dataset) -> float:
         # Calculate distance in meters per degree
         meters_per_degree = np.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2) / long_offset
 
-        return meters_per_degree
+        return meters_per_degree  # type: ignore[no-any-return]
     else:
         # For projected coordinates, return the linear unit conversion factor
-        return linear_units
+        return linear_units  # type: ignore[no-any-return]
 
 
-def feet_to_cell_count(feet: float, raster_path: str):
+def feet_to_cell_count(feet: float, raster_path: str) -> int:
     """
     Convert feet to cell count based on the cell size of a raster.
 
@@ -798,7 +808,7 @@ def feet_to_cell_count(feet: float, raster_path: str):
 SQ_MILES_TO_SQ_METERS = 2589988.11
 
 
-def sqmi_to_cell_count(sqmi: float, raster_path: str):
+def sqmi_to_cell_count(sqmi: float, raster_path: str) -> int:
     """
     Convert square miles to cell count based on the area of a raster.
 
